"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
require("buffer-safe");
var dgram = require("dgram");
var EventEmitter = require("events");
var os = require("os");
var padStart = require("pad-start");
var devices_1 = require("./devices");
var Broadlink = /** @class */ (function (_super) {
    __extends(Broadlink, _super);
    function Broadlink() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.devices = new Map();
        return _this;
    }
    Broadlink.prototype.discover = function () {
        var _this = this;
        var interfaces = os.networkInterfaces();
        var addresses = [];
        Object.keys(interfaces).forEach(function (idx) {
            var filteredInterface = interfaces[idx].find(function (iface) {
                return iface.family === 'IPv4' && iface.internal === false;
            });
            if (filteredInterface) {
                addresses.push(filteredInterface.address);
            }
        });
        addresses.forEach(function (addr) {
            var address = addr.split('.');
            var dgc = dgram.createSocket({ type: 'udp4', reuseAddr: true });
            dgc.on('listening', function () {
                dgc.setBroadcast(true);
                var port = dgc.address().port;
                var now = new Date();
                var timezone = now.getTimezoneOffset() / -3600;
                var year = now.getFullYear();
                var packet = Buffer.alloc(0x30, 0);
                if (timezone < 0) {
                    packet[0x08] = 0xff + timezone - 1;
                    packet[0x09] = 0xff;
                    packet[0x0a] = 0xff;
                    packet[0x0b] = 0xff;
                }
                else {
                    packet[0x08] = timezone;
                    packet[0x09] = 0;
                    packet[0x0a] = 0;
                    packet[0x0b] = 0;
                }
                packet[0x0c] = year & 0xff;
                packet[0x0d] = year >> 8;
                packet[0x0e] = now.getMinutes();
                packet[0x0f] = now.getHours();
                var subyear = year % 100;
                packet[0x10] = subyear;
                packet[0x11] = now.getDay();
                packet[0x12] = now.getDate();
                packet[0x13] = now.getMonth();
                packet[0x18] = parseInt(address[0], 10);
                packet[0x19] = parseInt(address[1], 10);
                packet[0x1a] = parseInt(address[2], 10);
                packet[0x1b] = parseInt(address[3], 10);
                packet[0x1c] = port & 0xff;
                packet[0x1d] = port >> 8;
                packet[0x26] = 6;
                var checksum = 0xbeaf;
                for (var i = 0; i < packet.length; i++) {
                    checksum += packet[i];
                }
                checksum = checksum & 0xffff;
                packet[0x20] = checksum & 0xff;
                packet[0x21] = checksum >> 8;
                dgc.send(packet, 0, packet.length, 80, '255.255.255.255');
            });
            "";
            dgc.on('message', function (msg, rinfo) {
                var host = rinfo;
                var macArr = [];
                for (var x = 58; x < (58 + 6); x++) {
                    macArr.push(padStart(msg.readUInt8(x).toString(16), 2, 0));
                }
                var mac = macArr.reverse().join(':');
                // const mac = Buffer.alloc(6,0);
                // msg.copy(mac, 0, 0x34, 0x40);
                var devtype = msg[0x34] | msg[0x35] << 8;
                if (!_this.devices[mac]) {
                    var dev_1 = _this.genDevice(devtype, host, mac);
                    _this.devices[mac] = dev_1;
                    dev_1.on('deviceReady', function () { _this.emit('deviceReady', dev_1); });
                    dev_1.auth();
                }
            });
            dgc.bind();
        });
    };
    Broadlink.prototype.genDevice = function (devtype, host, mac) {
        switch (devtype) {
            case 0:// SP1
                return new devices_1.SP1(host, mac);
            case 0x2711: // SP2
            case 0x27199 || 0x7919 || 0x271a || 0x791a: // Honeywell SP2
            case 0x2720: // SPMini
            case 0x2733 || 0x273e: // OEM branded SPMini
            case 0x2728: // SPMini2
            case 0x753e: // SP3
            case 0x2736:// SPMiniPlus
                return new devices_1.SP2(host, mac);
            case 0x4EB5:// MP1
                return new devices_1.MP1(host, mac);
            case 0x2712: // RM2
            case 0x2737: // RM Mini
            case 0x273d: // RM Pro Phicomm
            case 0x2783: // RM2 Home Plus
            case 0x277c: // RM2 Home Plus GDT
            case 0x272a: // RM2 Pro Plus
            case 0x2787: // RM2 Pro Plus 2
            case 0x278b: // RM2 Pro Plus BL
            case 0x278f:// RM Mini Shate
                return new devices_1.RM(host, mac);
            case 0x2714:// A1
                return new devices_1.A1(host, mac);
            default:
                if (devtype >= 0x7530 && devtype <= 0x273e) {
                    return new devices_1.SP2(host, mac);
                }
                return new devices_1.BaseDevice(host, mac);
        }
    };
    return Broadlink;
}(EventEmitter));
exports.Broadlink = Broadlink;
//# sourceMappingURL=Broadlink.js.map