"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var BaseDevice_1 = require("./BaseDevice");
var MP1 = /** @class */ (function (_super) {
    __extends(MP1, _super);
    function MP1(host, mac, timeout) {
        if (timeout === void 0) { timeout = 10; }
        var _this = _super.call(this, host, mac, timeout) || this;
        _this.type = 'MP1';
        _this.on('payload', function (err, data) {
            if (err) {
                _this.emit('error', err);
            }
            var state = {
                mac: _this.mac,
                host: _this.host,
                status: {
                    s1: Boolean(data[0x0e] & 0x01) ? 'on' : 'off',
                    s2: Boolean(data[0x0e] & 0x02) ? 'on' : 'off',
                    s3: Boolean(data[0x0e] & 0x04) ? 'on' : 'off',
                    s4: Boolean(data[0x0e] & 0x08) ? 'on' : 'off',
                },
            };
            _this.emit('state', state);
        });
        return _this;
    }
    /**
     * Sets the power state of the smart power strip.
     * @param sid_mask
     * @param state
     */
    MP1.prototype.setPowerMask = function (sidMask, state) {
        var packet = Buffer.alloc(16, 0);
        packet[0x00] = 0x0d;
        packet[0x02] = 0xa5;
        packet[0x03] = 0xa5;
        packet[0x04] = 0x5a;
        packet[0x05] = 0x5a;
        packet[0x06] = 0xb2 + (state ? (sidMask << 1) : sidMask);
        packet[0x07] = 0xc0;
        packet[0x08] = 0x02;
        packet[0x0a] = 0x03;
        packet[0x0d] = sidMask;
        packet[0x0e] = state ? sidMask : 0;
        this.sendPacket(0x6a, packet);
    };
    /**
     * Sets the power state of the smart power strip.
     */
    MP1.prototype.setPower = function (sid, state) {
        var sidMask = 0x01 << (sid - 1);
        // console.log(0xb2 + (state ? (sidMask << 1) : sidMask));
        this.setPowerMask(sidMask, state);
    };
    MP1.prototype.setState = function (state) {
        var _a = state.socketId.match(/^s(\d)$/), sid = _a[1];
        this.setPower(sid, state.status === 'on');
    };
    /**
     * Returns the power state of the smart power strip in raw format
     */
    MP1.prototype.requestState = function () {
        var packet = Buffer.alloc(16, 0);
        packet[0x00] = 0x0a;
        packet[0x02] = 0xa5;
        packet[0x03] = 0xa5;
        packet[0x04] = 0x5a;
        packet[0x05] = 0x5a;
        packet[0x06] = 0xae;
        packet[0x07] = 0xc0;
        packet[0x08] = 0x01;
        this.sendPacket(0x6a, packet);
    };
    return MP1;
}(BaseDevice_1.BaseDevice));
exports.MP1 = MP1;
//# sourceMappingURL=MP1.js.map